#include "gx_api.h"
#include "gx_on_ili9225_8bpp.h"
#include "sf_ili9225.h"


#define UNUSED_PARAMETER(a)             (a = a)
#define PALETTE_LENGTH                  256

static sf_ili9225_t* p_controller = NULL;
static uint16_t x_size = 0;
static uint16_t y_size = 0;
static uint32_t total_area = 0;
static uint16_t palette_565rgb[PALETTE_LENGTH];

static void convert_8bpp_to_565rgb( uint8_t* data_in,
                                    uint16_t* data_out,
                                    uint32_t length );

UINT _gx_synergy_display_driver_setup(GX_DISPLAY *display);
VOID _gx_display_driver_8bit_palette_assign(GX_DISPLAY *display, GX_COLOR *palette, INT count);
VOID _gx_synergy_buffer_toggle_8bpp(GX_CANVAS *canvas, GX_RECTANGLE *dirty);
VOID _gx_display_driver_8bit_palette_setup( GX_DISPLAY *display,
                                            VOID *aux_data,
                                            VOID (*toggle_function)(struct GX_CANVAS_STRUCT *canvas, GX_RECTANGLE *dirty_area));

void gx_on_ili9225_8bpp_init( sf_ili9225_t* p_target,
                              uint16_t row_len,
                              uint16_t row_num )
{
    p_controller = p_target;
    x_size = row_len;
    y_size = row_num;
    total_area = (uint32_t) (row_len * row_num);
}

UINT gx_on_ili9225_8bpp_setup(GX_DISPLAY * p_display)
{
    _gx_synergy_display_driver_setup( p_display );
    return GX_SUCCESS;
}

VOID _gx_display_driver_8bit_palette_assign(GX_DISPLAY *p_display, GX_COLOR *p_palette, INT count)
{
    uint16_t i;
    UNUSED_PARAMETER( p_display );
    UNUSED_PARAMETER( count );
    
    /* Create 565rbg reversed palette */
    for( i=0 ; i<PALETTE_LENGTH ; i++ )
    {
        uint32_t raw = p_palette[i];
        palette_565rgb[i] = (uint16_t) __REV16(((raw & 0xF80000) >> 8) |((raw & 0x00FC00) >> 5) |((raw & 0x0000FF) >> 3));
    }
}

VOID _gx_synergy_buffer_toggle_8bpp(GX_CANVAS *p_canvas, GX_RECTANGLE *p_region)
{
    uint16_t i;
    uint16_t row[x_size];   /* It will contain a single 16bit 565rgb row */
    uint16_t actual_row = (uint16_t)(p_region->gx_rectangle_right - p_region->gx_rectangle_left + 1);
                            /* This is the actual "dirty" row length */
    uint32_t start_id;      /* Working index in the dirty 8bpp buffer */
    /* Iterate on all the rows */
    for( i=(uint16_t)p_region->gx_rectangle_top ; i<=(uint16_t)p_region->gx_rectangle_bottom ; i++ )
    {
        start_id = (uint32_t) ((i*x_size) + p_region->gx_rectangle_left); /* Select the current row to convert and refresh */
        
        /* Convert the 8bpp data into 16bpp */
        convert_8bpp_to_565rgb( &((uint8_t*)p_canvas->gx_canvas_memory)[start_id],
                                &row[0],
                                actual_row );
        
        /* Draw the current row */
        p_controller->p_api->drawPartial( p_controller->p_ctrl,
                                          (uint8_t*) &row[0],
                                          (uint16_t) p_region->gx_rectangle_left,
                                          i,
                                          actual_row,
                                          1 );
    }
}

static void convert_8bpp_to_565rgb( uint8_t* data_in,
                                    uint16_t* data_out,
                                    uint32_t length )
{
    uint32_t i;
    
    for( i=0; i<length ; i++ )
    {
        data_out[i] = palette_565rgb[data_in[i]];
    }
}
