/***********************************************************************************************************************
 * Copyright [2018] RELOC s.r.l. - All rights strictly reserved
 * This Software is provided for evaluation purposes; any other use — including reproduction, modification, use for
 * a commercial product, distribution, or republication — without the prior written permission of the Copyright holder
 * is strictly prohibited. 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
 * OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 **********************************************************************************************************************/

/**********************************************************************************************************************
 * File Name    : sf_ili9225.h
 * Description  : ILI9225 TFT LCD Single Chip Driver.
 **********************************************************************************************************************/

#ifndef SF_ILI9225_H
#define SF_ILI9225_H

/***********************************************************************************************************************
 * Includes
 **********************************************************************************************************************/
#include "hal_data.h"

/**********************************************************************************************************************
 * Macro definitions
 **********************************************************************************************************************/
#define SF_ILI9225_CODE_VERSION_MAJOR (0)
#define SF_ILI9225_CODE_VERSION_MINOR (1)


#define LCD_CMD     (IOPORT_PORT_01_PIN_06)
#define LCD_RESET   (IOPORT_PORT_01_PIN_05)
#define LCD_CS      (IOPORT_PORT_01_PIN_04)

/**********************************************************************************************************************
 * Typedef definitions
 *********************************************************************************************************************/
/** Channel control block. DO NOT INITIALIZE.  Initialization occurs when SF_ILI9225_Open is called */
typedef struct st_sf_ili9225_ctrl
{
    uint32_t                    open;                   ///< Used by driver to check if control block is valid
    ioport_port_pin_t           reset_pin;
    ioport_port_pin_t           cmd_pin;
    const sf_spi_instance_t *   spi;                    ///< SPI Framework Device
    uint16_t                    x_size;
    uint16_t                    y_size;
    uint8_t                     byte_per_pixel;
} sf_ili9225_ctrl_t;

/** Configuration for RTOS integrated ILI9225 Driver. */
typedef struct st_sf_ili9225_cfg
{
    ioport_port_pin_t           reset_pin;
    ioport_port_pin_t           cmd_pin;
    const sf_spi_instance_t *   spi;                    ///< SPI Framework Device
    uint16_t                    x_size;
    uint16_t                    y_size;
    uint8_t                     byte_per_pixel;
    uint16_t                    rotation_angle;
} sf_ili9225_cfg_t;

typedef struct st_sf_ili9225_api
{
    ssp_err_t (* open)(sf_ili9225_ctrl_t * p_ctrl,
                       sf_ili9225_cfg_t const * const p_cfg);
    
    ssp_err_t (* close)(sf_ili9225_ctrl_t * p_ctrl);
    
    ssp_err_t (* drawFull)( sf_ili9225_ctrl_t * p_ctrl,
                            uint8_t* p_src );
    
    ssp_err_t (* drawPartial)( sf_ili9225_ctrl_t * const p_ctrl,
                               uint8_t * p_src,
                               uint16_t x_offset,
                               uint16_t y_offset,
                               uint16_t row_len,
                               uint16_t rows );
} sf_ili9225_api_t;

typedef struct st_sf_ili9225
{
    sf_ili9225_ctrl_t* p_ctrl;
    sf_ili9225_cfg_t* p_cfg;
    sf_ili9225_api_t* p_api;
} sf_ili9225_t;


/**********************************************************************************************************************
 * Exported global variables
 **********************************************************************************************************************/
extern sf_ili9225_api_t sf_ili9225_api;

#endif /* SF_ILI9225_H */
